/* SPIM S20 MIPS simulator.
   Definitions for the SPIM S20.
   Copyright (C) 1990 by James Larus (larus@cs.wisc.edu).

   SPIM is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 1, or (at your option) any
   later version.

   SPIM is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.

   You should have received a copy of the GNU General Public License
   along with GNU CC; see the file COPYING.  If not, write to James R.
   Larus, Computer Sciences Department, University of Wisconsin--Madison,
   1210 West Dayton Street, Madison, WI 53706, USA or to the Free
   Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */


/* $Header: /var/home/larus/SPIM/RCS/spim.h,v 3.6 1991/05/09 21:00:44 larus Exp larus $
*/


#ifndef NULL
#define NULL 0
#endif

#define streq(s1, s2) !strcmp(s1, s2)


/* Round V to next greatest B boundary */

#define ROUND(V, B) (((int) V + (B-1)) & ~(B-1))


/* Sign-extend a short to a long */

#define SIGN_EX(X)	((X) & 0x8000 ? (X) | 0xffff0000 : (X))


/* Useful and pervasive declarations: */

#ifndef __linux__
double atof ();
int atoi ();
void bzero ();
void bcopy ();
int fclose ();
int fprintf ();
char *malloc ();
int printf ();
int scanf ();
int strcmp ();
char *strcpy ();
char *strncpy ();
#endif
int run_error ();
void write_output ();
void error ();
void fatal_error ();

/* Variables: */

int bare_machine;		/* Simulate bare instruction set */
int quiet;			/* No warning messages */
int source_file;		/* Program is source, not binary */


int message_out, console_out;	/* Real type depends on X/terminal interface */



/* Type of a memory address. */

typedef unsigned long mem_addr;


#define BYTES_PER_WORD 4	/* On the MIPS processor */


/* Initial size of memory segments.  The segments expand as needed. */

#ifndef TEXT_SIZE
#define TEXT_SIZE	524288	/* 1/2 MB */
#endif


/* The data segment must be larger than 64K since we immediate grab 64K for
 the small data segment pointed to by $gp. */

#ifndef DATA_SIZE
#define DATA_SIZE	524288	/* 1/2 MB */
#endif

#ifndef K_DATA_SIZE
#define K_DATA_SIZE	65536	/* 64 KB */
#endif


#ifndef STACK_SIZE
#define STACK_SIZE	65536	/* 64 KB */
#endif


#ifndef K_TEXT_SIZE
#define K_TEXT_SIZE	65536	/* 64 KB */
#endif


/* Maximum amount that we are allowed to grow a segment when an
   out-of-bounds memory reference causes expansion. */

#define MAX_SEG_GROWTH 65536	/* 64 KB */


/* Name of the function to invoke at start up */

#define DEFAULT_RUN_LOCATION "__start"


/* Default number of instructions to execute. */

#define DEFAULT_RUN_STEPS 10000000


/* Address to branch to when exception occurs */

#define EXCEPTION_ADDR 0x80000080


/* Maximum size of object stored in the small data segment pointed to by
   $gp */

#define SMALL_DATA_SEG_MAX_SIZE 8



/* Argument passing registers */

#define REG_V0 2
#define REG_A0 4
#define REG_A1 5
#define REG_A2 6
#define REG_A3 7
#define REG_FA0 12


/* Result registers */

#define REG_RES 2
#define REG_FRES 0


/* $gp registers */

#define REG_GP 28

#define K 1024



/* Triple containing a string and two integers.	 Used in tables
   mapping from a name to values. */

typedef struct strint
{
  char *name;
  int value1;
  int value2;
} inst_info;


inst_info *map_int_to_inst_info ();
inst_info *map_string_to_inst_info ();
